﻿
// Define this symbol if you want the display to pause after a while
// so you can take screenshots
#define SREENSHOT_PAUSE

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Kinect;
using KinectUtils;

namespace KinectMgrDemo
{

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

#if SREENSHOT_PAUSE

        int trackCount = 0;
        int trackLimit = 300;

#endif

        #region Kinect setup

        KinectManager myKinectMgr;
        KinectSensor myKinect;

        void setupKinect()
        {
            try
            {


                myKinectMgr = new KinectManager(0);
                myKinect = myKinectMgr.Kinect;

                myKinectMgr.DoProcessSkeletons +=
                    new KinectManager.ProcessSkeletons(processSkeletonFrame);

                myKinectMgr.DoProcessDepthFrame +=
                    new KinectManager.ProcessDepthFrame(processDepthFrame);

                myKinectMgr.DoProcessVideoFrame +=
                    new KinectManager.ProcessVideoFrame(processVideoFrame);

                myKinectMgr.DoStatusChanged +=
                    new KinectManager.StatusChanged(updateStatus);

                myKinectMgr.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Kinect Demo");
                Application.Current.Shutdown();
            }
        }

        void shutdownKinect()
        {
            if (myKinectMgr != null)
                myKinectMgr.Stop();
        }

        #endregion

        #region Skeleton Depth Display

        short[] depthData = null;
        byte[] depthColorImage = null;
        int depthWidth;
        int depthHeight;
        WriteableBitmap depthImageBitmap = null;

        void processDepthFrame(DepthImageFrame depthFrame)
        {

#if SREENSHOT_PAUSE
            if (trackCount == trackLimit) return;
#endif

            if (depthColorImage == null)
            {
                depthColorImage = new byte[depthFrame.Width * depthFrame.Height * 4];
            }

            if (depthData == null)
                depthData = new short[depthFrame.Width * depthFrame.Height];

            depthFrame.CopyPixelDataTo(depthData);

            int depthColorImagePos = 0;

            for (int depthPos = 0; depthPos < depthData.Length; depthPos++)
            {
                int depthValue = depthData[depthPos];

                int playerNo = depthValue & DepthImageFrame.PlayerIndexBitmask;
                depthValue = depthValue >> DepthImageFrame.PlayerIndexBitmaskWidth;

                byte depthByte = (byte)(255 - (depthValue >> 4));

                if (playerNo != 0)
                {
                    // got a player 
                    // Convert to playerno in the array
                    playerNo--;

                    Color playerCol = skeletonColors[playerNo];
                    // Blue
                    depthColorImage[depthColorImagePos] = (byte)(playerCol.B / 2);
                    depthColorImagePos++;
                    // Green
                    depthColorImage[depthColorImagePos] = (byte)(playerCol.G / 2);
                    depthColorImagePos++;
                    // Red
                    depthColorImage[depthColorImagePos] = (byte)(playerCol.R / 2);
                    depthColorImagePos++;
                    // transparency
                    depthColorImagePos++;
                }
                else
                {
                    // Blue
                    depthColorImage[depthColorImagePos] = depthByte;
                    depthColorImagePos++;
                    // Green
                    depthColorImage[depthColorImagePos] = depthByte;
                    depthColorImagePos++;
                    // Red
                    depthColorImage[depthColorImagePos] = depthByte;
                    depthColorImagePos++;
                    // transparency
                    depthColorImagePos++;
                }

                depthColorImage[100] = 0;
            }

            depthWidth = depthFrame.Width;
            depthHeight = depthFrame.Height;

            Dispatcher.Invoke(new Action(() => drawDepthFrame()));
        }

        void drawDepthFrame()
        {
            if (depthColorImage == null) return;

            // we now have a new array of color data

            if (depthImageBitmap == null)
            {
                this.depthImageBitmap = new WriteableBitmap(
                    depthWidth,
                    depthHeight,
                    96,  // DpiX
                    96,  // DpiY
                    PixelFormats.Bgr32,
                    null);
                kinectDepth.Source = depthImageBitmap;
            }

            this.depthImageBitmap.WritePixels(
                new Int32Rect(0, 0, depthWidth, depthHeight),
                depthColorImage, // video data
                depthWidth * 4, // stride,
                0   // offset into the array - start at 0
                );
        }


        #endregion

        #region myKinectMgr Skeleton Display

        Skeleton[] skeletons = null;

        void processSkeletonFrame(SkeletonFrame frame)
        {

#if SREENSHOT_PAUSE
            if (trackCount == trackLimit) return;
            trackCount++;
#endif

            skeletons = new Skeleton[frame.SkeletonArrayLength];
            frame.CopySkeletonDataTo(skeletons);
            Dispatcher.Invoke(new Action(() => drawSkeleton()));
        }

        Color[] skeletonColors = new[] {
            Colors.Red, Colors.Green, Colors.Blue, Colors.Yellow,Colors.Purple, Colors.Cyan};

        void drawSkeleton()
        {
            if (skeletons == null) return;

            skeletonCanvas.Children.Clear();

            for (int i = 0; i < 6; i++)
            {
                Skeleton skeleton = skeletons[i];

                if (skeleton.TrackingState != SkeletonTrackingState.Tracked)
                    continue;
                skeletonBrush = new SolidColorBrush(skeletonColors[i]);
                drawSkeleton(skeleton);
            }
        }

        Brush skeletonBrush = new SolidColorBrush(Colors.Red);

        void addLine(Joint j1, Joint j2)
        {
            Line boneLine = new Line();
            boneLine.Stroke = skeletonBrush;
            boneLine.StrokeThickness = 5;

            DepthImagePoint j1P = myKinect.MapSkeletonPointToDepth(j1.Position, DepthImageFormat.Resolution640x480Fps30);
            boneLine.X1 = j1P.X;
            boneLine.Y1 = j1P.Y;

            DepthImagePoint j2P = myKinect.MapSkeletonPointToDepth(j2.Position, DepthImageFormat.Resolution640x480Fps30);
            boneLine.X2 = j2P.X;
            boneLine.Y2 = j2P.Y;

            skeletonCanvas.Children.Add(boneLine);
        }

        void drawSkeleton(Skeleton skeleton)
        {
            // Spine
            addLine(skeleton.Joints[JointType.Head], skeleton.Joints[JointType.ShoulderCenter]);
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.Spine]);

            // Left leg
            addLine(skeleton.Joints[JointType.Spine], skeleton.Joints[JointType.HipCenter]);
            addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipLeft]);
            addLine(skeleton.Joints[JointType.HipLeft], skeleton.Joints[JointType.KneeLeft]);
            addLine(skeleton.Joints[JointType.KneeLeft], skeleton.Joints[JointType.AnkleLeft]);
            addLine(skeleton.Joints[JointType.AnkleLeft], skeleton.Joints[JointType.FootLeft]);

            // Right leg
            addLine(skeleton.Joints[JointType.HipCenter], skeleton.Joints[JointType.HipRight]);
            addLine(skeleton.Joints[JointType.HipRight], skeleton.Joints[JointType.KneeRight]);
            addLine(skeleton.Joints[JointType.KneeRight], skeleton.Joints[JointType.AnkleRight]);
            addLine(skeleton.Joints[JointType.AnkleRight], skeleton.Joints[JointType.FootRight]);

            // Left arm
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderLeft]);
            addLine(skeleton.Joints[JointType.ShoulderLeft], skeleton.Joints[JointType.ElbowLeft]);
            addLine(skeleton.Joints[JointType.ElbowLeft], skeleton.Joints[JointType.WristLeft]);
            addLine(skeleton.Joints[JointType.WristLeft], skeleton.Joints[JointType.HandLeft]);

            // Right arm
            addLine(skeleton.Joints[JointType.ShoulderCenter], skeleton.Joints[JointType.ShoulderRight]);
            addLine(skeleton.Joints[JointType.ShoulderRight], skeleton.Joints[JointType.ElbowRight]);
            addLine(skeleton.Joints[JointType.ElbowRight], skeleton.Joints[JointType.WristRight]);
            addLine(skeleton.Joints[JointType.WristRight], skeleton.Joints[JointType.HandRight]);

        }

        #endregion

        #region Video Display

        byte[] colorData = null;
        int frameWidth;
        int frameHeight;

        WriteableBitmap colorImageBitmap = null;

        void processVideoFrame(ColorImageFrame colorFrame)
        {

#if SREENSHOT_PAUSE
            if (trackCount == trackLimit) return;
#endif
            if (colorData == null)
            {
                colorData = new byte[colorFrame.PixelDataLength];
                frameWidth = colorFrame.Width;
                frameHeight = colorFrame.Height;
            }

            colorFrame.CopyPixelDataTo(colorData);

            Dispatcher.Invoke(new Action(() => drawDisplay()));
        }

        void drawDisplay()
        {
            if (colorImageBitmap == null)
            {
                this.colorImageBitmap = new WriteableBitmap(
                    frameWidth,
                    frameHeight,
                    96,  // DpiX
                    96,  // DpiY
                    PixelFormats.Bgr32,
                    null);
            }

            this.colorImageBitmap.WritePixels(
                new Int32Rect(0, 0, frameWidth, frameHeight),
                colorData, // video data
                frameWidth * 4, // stride,
                0   // offset into the array - start at 0
                );

            kinectVideo.Source = colorImageBitmap;
        }

        #endregion

        #region Status Display

        string newStatus;

        void updateStatus(string status)
        {
            newStatus = status;
            Dispatcher.Invoke(new Action(() => updateMessage()));
        }

        void updateMessage()
        {
            statusMessage.Text = newStatus;
        }

        #endregion

        #region Window Event handlers

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            setupKinect();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            shutdownKinect();
        }

        #endregion
    }
}
